<!--

//************************************************************************************************************
/* INITIALIZATION SECTION BEGIN */
var rows = 9;
var cols = 9;

var sudokuTableId = 'tab';
var enableHints = false;

var backgroundColorForCellWithOnlyOneHint = '#00FFFF';
var defaultCellBackgroundColor = undefined;

var record = new Array(81); //record of the game
var moveNumber = 0; //current move pointer
var lastMoveNumber = moveNumber; //number of moves (used when moves are reversed and then replayed)

//initialize sudoku table
var sudoku = new Array(rows);
for (row = 0; row < rows; row++) {
	sudoku[row] = new Array(cols);
}
clearSudoku();
/* INITIALIZATION SECTION END */
//************************************************************************************************************
function switchHints() {
	//get default cell background color if undefined (from cell [0, 0]) - this should work only for first hints enabling
	if (defaultCellBackgroundColor == undefined)
		defaultCellBackgroundColor = document.getElementById(sudokuTableId).rows[0].cells[0].style.backgroundColor;

	if (enableHints == false)
		enableHints = true;
	else
		enableHints = false;

	showHints();
}
//************************************************************************************************************
function insertDigit(y, x, digit) {
	if (digit != 1 && digit != 2 && digit != 3 && digit != 4 && digit != 5 && digit != 6 && digit != 7 && digit != 8 && digit != 9)
		return false;

	sudoku[y][x] = digit;

	if (sudokuCorrect(y, x)) {
		//get default cell background color if undefined (from cell [0, 0]) - this should work only for first correct digit
		if (defaultCellBackgroundColor == undefined)
			defaultCellBackgroundColor = document.getElementById(sudokuTableId).rows[0].cells[0].style.backgroundColor;

		writeDownMove(y, x, digit);
		resetCellStyle(y, x);
		setDigitOnly(y, x, digit);
		showHints();

		return true;
	}
	else {
		deleteDigit(y, x);
		return false;
	}
}
//************************************************************************************************************
function testDigit(y, x, digit) {
	sudoku[y][x] = digit;
	var result = sudokuCorrect(y, x);
	deleteDigit(y, x);

	return result;
}
//************************************************************************************************************
function deleteDigit(y, x) {
	sudoku[y][x] = '';
}
//************************************************************************************************************
function sudokuCorrect(y, x) {
	var row, col;

	var rowArray = new Array(0, 0, 0, 0, 0, 0, 0, 0, 0);
	var colArray = new Array(0, 0, 0, 0, 0, 0, 0, 0, 0);
	var squareArray = new Array(0, 0, 0, 0, 0, 0, 0, 0, 0);
	var digit;

	var diff;
	var xSquare;
	var ySquare;

	//check row
	for (col = 0; col < cols; col++) {
		digit = sudoku[y][col];
		rowArray[digit - 1]++;
	}
	for (i = 0; i < rowArray.length; i++)
		if (rowArray[i] > 1)
			return false;

	//check column
	for (row = 0; row < rows; row++) {
		digit = sudoku[row][x];
		colArray[digit - 1]++;
	}
	for (i = 0; i < colArray.length; i++)
		if (colArray[i] > 1)
			return false;

	//check 3x3 square
	//a) find vertical square number
	ySquare = Math.floor(y / 3);
	//b) find horizontal square number
	xSquare = Math.floor(x / 3);
	//c) check square integrity
	for (row = 3 * ySquare; row < 3 * ySquare + 3; row++)
		for (col = 3 * xSquare; col < 3 * xSquare + 3; col++) {
			digit = sudoku[row][col];
			squareArray[digit - 1]++
		}
  for (i = 0; i < squareArray.length; i++)
  if (squareArray[i] > 1)
    return false;

	return true;
}
//************************************************************************************************************
function clearHints() {
  var row, col;

  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++)
      if (!getEditReadOnly(row, col)) {
				setHintText(row, col, '');
				resetCellStyle(row, col);
			}
}
//************************************************************************************************************
function showHints() {
	clearHints();

	if (enableHints) {
	  var row, col, digit;

	  for (row = 0; row < rows; row++)
	    for (col = 0; col < cols; col++)
	      if (!getEditReadOnly(row, col)) {
	        for (digit = 1; digit <= 9; digit++)
	          if (testDigit(row, col, digit))
							setHintText(row, col, getHintText(row, col) + digit);

					if (getHintText(row, col).length == 1) //obvious move - only one hint
						distinguishCellWithOnlyOneHint(row, col);
				}
	}
}
//************************************************************************************************************
function setDigitOnly(y, x, digit) {
	setHintText(y, x, '');
	setEditText(y, x, digit);
	setEditReadOnly(y, x, true);
}
//************************************************************************************************************
function writeDownMove(y, x, digit) {
	record[moveNumber++] = new Array(y, x, digit);
	lastMoveNumber = moveNumber;
}
//************************************************************************************************************
function gotoNextMove() {
	if (moveNumber < lastMoveNumber) {
	  var y = record[moveNumber][0];
	  var x = record[moveNumber][1];
		var digit = record[moveNumber][2];
		moveNumber++;

		sudoku[y][x] = digit;

		resetCellStyle(y, x);
		setDigitOnly(y, x, digit);

		showHints();
	}
}
//************************************************************************************************************
function gotoPreviousMove() {
	if (moveNumber > 0) {
	  moveNumber--;
	  var y = record[moveNumber][0];
	  var x = record[moveNumber][1];

	  deleteDigit(y, x);

		setEditReadOnly(y, x, false);
		setEditText(y, x, '');

	  showHints();
	}
}
//************************************************************************************************************
function getHintText(y, x) {
	return document.getElementById(sudokuTableId).rows[y].cells[x].childNodes[0].nodeValue;
}
//************************************************************************************************************
function setHintText(y, x, hintText) {
	document.getElementById(sudokuTableId).rows[y].cells[x].childNodes[0].nodeValue = hintText;
}
//************************************************************************************************************
function getEditReadOnly(y, x) {
	return document.getElementById(sudokuTableId).rows[y].cells[x].childNodes[1].readOnly;
}
//************************************************************************************************************
function setEditReadOnly(y, x, editReadOnly) {
	document.getElementById(sudokuTableId).rows[y].cells[x].childNodes[1].readOnly = editReadOnly;
}
//************************************************************************************************************
/*
function getEditText(y, x) {
	return document.getElementById(sudokuTableId).rows[y].cells[x].childNodes[1].value;
}
*/
//************************************************************************************************************
function setEditText(y, x, editText) {
	document.getElementById(sudokuTableId).rows[y].cells[x].childNodes[1].value = editText;
}
//************************************************************************************************************
function distinguishCellWithOnlyOneHint(y, x) {
	document.getElementById(sudokuTableId).rows[y].cells[x].style.backgroundColor = backgroundColorForCellWithOnlyOneHint;
}
//************************************************************************************************************
function resetCellStyle(y, x) {
		document.getElementById(sudokuTableId).rows[y].cells[x].style.backgroundColor = defaultCellBackgroundColor;
}
//************************************************************************************************************
function clearSudoku() {
	var row, col;

	for (row = 0; row < rows; row++)
		for (col = 0; col < cols; col++)
			sudoku[row][col] = '';
}
//************************************************************************************************************
function clearHTMLTable() {
	var row, col;

  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++) {
			document.getElementById(sudokuTableId).rows[row].cells[col].childNodes[0].nodeValue = '';
			document.getElementById(sudokuTableId).rows[row].cells[col].childNodes[1].readOnly = false;
			document.getElementById(sudokuTableId).rows[row].cells[col].childNodes[1].value = '';
		}
}
//************************************************************************************************************
function resetAll() {
	clearSudoku();
	clearHTMLTable();

	enableHints = false;
	defaultCellBackgroundColor = undefined;
	record = new Array(81);
	moveNumber = 0;
	lastMoveNumber = moveNumber;

	if (arrangeDigits)
		insertDigits();
}
//************************************************************************************************************
//-->